﻿@bypass-integration @feature:1279
Feature: Upgrade configuration from v3.4 to v5

Rule: Updates scema and compatibility version

@tc:1399
Scenario: Updates scema and compatibility version
	Given there is an Azure DevOps project
	And there is a file specsync.json as:
		"""
		{
		  "$schema": "https://schemas.specsolutions.eu/specsync4azuredevops-config-latest.json",
		  "compatibilityVersion": "3.4",
		
		  "remote": {
		    "projectUrl": "[remote-project-url]"
		  }
		}
		"""
	When SpecSync upgrade is executed with answers successfully:
		| question    | answer |
		| Is this ok? | yes    |
	Then the configuration file should be updated with at least
		| setting              | value                          |
		| compatibilityVersion | [config-compatibility-version] |
		| $schema              | [config-schema-url]            |

@adoSpecific
Rule: Upgrades old-style remote suite

Background:
	Given there is an Azure DevOps project
	And there is a file specsync.json as:
		"""
		{
		  "compatibilityVersion": "3.4",
		
		  "remote": {
		    "projectUrl": "[remote-project-url]",
		    "testSuite": { 
		      "name": "My Suite",
		      "testPlan": "My Plan"
		    }
		  }
		}
		"""

@tc:1400
Scenario: Updates remote suite to remote scope
	When SpecSync upgrade is executed with answers successfully:
		| question                                               | answer                                            |
		| How to upgrade the existing 'remote/testSuite' setting | Keep the existing Test Suite remote scope setting |
	Then the configuration file should be updated with at least
		| setting                | value                                         |
		| remote/scope/type      | testSuite                                     |
		| remote/scope/testSuite | { "name": "My Suite", "testPlan": "My Plan" } |
		| remote/testSuite       | [deleted]                                     |

@tc:1401
Scenario: Converts remote suite to hierarchy
	When SpecSync upgrade is executed with answers successfully:
		| question                                               | answer                                                            |
		| How to upgrade the existing 'remote/testSuite' setting | Convert 'remote/testSuite' to a synchronized Test Suite hierarchy |
	Then the configuration file should be updated with at least
		| setting             | value                                         |
		| hierarchies[0]/name | My Suite                                      |
		| hierarchies[0]/type | single                                        |
		| hierarchies[0]/node | { "name": "My Suite", "testPlan": "My Plan" } |
		| remote/testSuite    | [deleted]                                     |

@tc:1402
Scenario: Removes remote suite
	When SpecSync upgrade is executed with answers successfully:
		| question                                               | answer                                 |
		| How to upgrade the existing 'remote/testSuite' setting | Remove the 'remote/testSuite' setting. |
	Then the configuration file should be updated with at least
		| setting          | value     |
		| remote/testSuite | [deleted] |


Rule: Upgrades publish test results configuration

@tc:1515
Scenario: Confirms attachment publish changes
	Given there is an Azure DevOps project
	And there is a file specsync.json as:
		"""
		{
		  "$schema": "https://schemas.specsolutions.eu/specsync4azuredevops-config-latest.json",
		  "compatibilityVersion": "3.4",
		
		  "remote": {
		    "projectUrl": "[remote-project-url]"
		  }
		}
		"""
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                              | answer |
		| Do you want to keep v3 behavior ('all') for publishing attachments for passing tests? | yes    |
	Then the configuration file should be updated with at least
		| setting                                              | value |
		| publishTestResults/publishAttachmentsForPassingTests | all   |

@adoSpecific
Rule: Converts AddTestCasesToSuites customization to hierarchy

Background:
	Given there is an Azure DevOps project
	And there is a file specsync.json as:
		"""
		{
		  "compatibilityVersion": "3.4",
		
		  "customizations": {
		    "addTestCasesToSuites": {
		      "enabled": true,
		      "testPlan": "My Plan",
		      "testSuites": [
		        {
		          "name": "Important Logic",
		          "condition": "@important and not @external"
		        },
		        {
		          "name": "Nice to have",
		          "condition": "@niceToHave"
		        }
		      ]
		    }
		  }
		}
		"""

@tc:1403
Scenario: Converts AddTestCasesToSuites customization to single custom hierarchy
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                  | answer                                                           |
		| How to upgrade the existing 'customizations/addTestCasesToSuites' setting | Convert Test Suite configurations to a single 'custom' hierarchy |
	Then the configuration file should be updated with at least
		| setting                              | value                                                                      |
		| customizations/addTestCasesToSuites  | [deleted]                                                                  |
		| hierarchies[0]/root/testPlan         | My Plan                                                                    |
		| hierarchies[0]/name                  | addTestCasesToSuites-suites                                                |
		| hierarchies[0]/type                  | custom                                                                     |
		| hierarchies[0]/ignoreAdditionalNodes | True                                                                       |
		| hierarchies[0]/nodes[]               | { "name": "Important Logic", "condition": "@important and not @external" } |
		| hierarchies[0]/nodes[]               | { "name": "Nice to have", "condition": "@niceToHave" }                     |

@tc:1404
Scenario: Converts AddTestCasesToSuites customization to multiple single hierarchy
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                  | answer                                                              |
		| How to upgrade the existing 'customizations/addTestCasesToSuites' setting | Convert Test Suite configurations to multiple single-node hierarchy |
	Then the configuration file should be updated with at least
		| setting                             | value                         |
		| customizations/addTestCasesToSuites | [deleted]                     |
		| hierarchies[0]/root/testPlan        | My Plan                       |
		| hierarchies[0]/name                 | addTestCasesToSuites-suites-1 |
		| hierarchies[0]/type                 | single                        |
		| hierarchies[0]/condition            | @important and not @external  |
		| hierarchies[0]/node                 | { "name": "Important Logic" } |
		| hierarchies[1]/root/testPlan        | My Plan                       |
		| hierarchies[1]/name                 | addTestCasesToSuites-suites-2 |
		| hierarchies[1]/type                 | single                        |
		| hierarchies[1]/condition            | @niceToHave                   |
		| hierarchies[1]/node                 | { "name": "Nice to have" }    |

@tc:1405
Scenario: Removes AddTestCasesToSuites customization
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                  | answer                                                   |
		| How to upgrade the existing 'customizations/addTestCasesToSuites' setting | Remove the 'customizations/addTestCasesToSuites' setting |
	Then the configuration file should be updated with at least
		| setting                             | value     |
		| customizations/addTestCasesToSuites | [deleted] |

@tc:1406
Scenario: Not enabled AddTestCasesToSuites customization is removed
	Given there is a file specsync.json as:
		"""
		{
		  "compatibilityVersion": "3.4",
		
		  "customizations": {
		    "addTestCasesToSuites": {
		      "enabled": false,
		      "testSuites": [
		        {
		          "name": "Important Logic",
		          "condition": "@important"
		        }
		      ]
		    }
		  }
		}
		"""
	When SpecSync upgrade is executed with answers successfully:
		| question                                                | answer |
		| Do you want to continue with removing the customization | yes    |
	Then the configuration file should be updated with at least
		| setting                             | value     |
		| customizations/addTestCasesToSuites | [deleted] |

Rule: Converts CustomFieldUpdates customization to hierarchy

Background:
	Given there is an Azure DevOps project
	And there is a file specsync.json as:
		"""
		{
		  "compatibilityVersion": "3.4",
		
		  "customizations": {
			"customFieldUpdates": {
			  "enabled": true,
			  "updates": {
				"Custom.FeatureName": "{feature-name}",
				"System.Description": "Description: {scenario-description}"
			  }
			}
		  }
		}
		"""

Scenario: Converts CustomFieldUpdates customization to field updates
	When SpecSync upgrade is executed with answers successfully:
		| question    | answer |
		| Is this ok? | yes    |
	Then the configuration file should be updated with at least
		| setting                                         | value                               |
		| customizations/customFieldUpdates               | [deleted]                           |
		| synchronization/fieldUpdates/Custom.FeatureName | {feature-name}                      |
		| synchronization/fieldUpdates/System.Description | Description: {scenario-description} |

Rule: Setup remote scope

Background:
	Given there is an Azure DevOps project
	And there is a file specsync.json as:
		"""
		{
		  "compatibilityVersion": "3.4",
		
		  "remote": {
			"projectUrl": "[remote-project-url]"
		  }
		}
		"""

@tc:1407
@adoSpecific
Scenario: Sets up a TestSuite remote scope for Azure DevOps
	When SpecSync upgrade is executed with answers successfully:
		| question                                | answer                                    |
		| Do you want to setup a remote scope?    | Yes, configure a 'testSuite' remote scope |
		| The name of the Test Plan for the suite | My Plan                                   |
		| The path of the suite within the plan   | My Suite                                  |
	Then the configuration file should be updated with at least
		| setting                         | value     |
		| remote/scope/type               | testSuite |
		| remote/scope/testSuite/path     | My Suite  |
		| remote/scope/testSuite/testPlan | My Plan   |

@tc:1408
@adoSpecific
Scenario: Sets up a ManagedQuery remote scope for Azure DevOps
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                                | answer                                       |
		| Do you want to set a configuration key now?                                             | no                                           |
		| Do you want to setup a remote scope?                                                    | Yes, configure a 'managedQuery' remote scope |
		| Please specify a key that identifies this configuration within the Azure DevOps project | MyConfig01                                   |
	Then the configuration file should be updated with at least
		| setting           | value        |
		| remote/scope/type | managedQuery |
		| configurationKey  | MyConfig01   |

@tc:1449
Scenario: Sets up a Tag remote scope
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                                | answer                              |
		| Do you want to set a configuration key now?                                             | no                                  |
		| Do you want to setup a remote scope?                                                    | Yes, configure a 'tag' remote scope |
		| Please specify a key that identifies this configuration within the Azure DevOps project | MyConfig01                          |
	Then the configuration file should be updated with at least
		| setting           | value      |
		| remote/scope/type | tag        |
		| configurationKey  | MyConfig01 |

@adoSpecific @feature:1388
Rule: Setup hierarchies

Background:
	Given there is an Azure DevOps project
	And there is a file specsync.json as:
		"""
		{
		  "compatibilityVersion": "3.4",
		
		  "remote": {
			"projectUrl": "[remote-project-url]"
		  }
		}
		"""

@tc:1409
Scenario: A folder hierarchy is configured
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                             | answer       |
		| Do you want to configure a hierarchy now                                             | yes          |
		| Which hierarchy type you would like to use?                                          | 'folders'    |
		| Please specify a name for the hierarchy                                              | MyHierarchy  |
		| Please specify the name of the Test Plan where the hierarchy should be created       | Plan1        |
		| Please specify the path of the root Test Suite where the hierarchy should be created | RootSuite    |
		| Please specify a folder prefix to skip in the hierarchy                              | src/Features |
	Then the configuration file should be updated with at least
		| setting                         | value        |
		| hierarchies[0]/name             | MyHierarchy  |
		| hierarchies[0]/type             | folders      |
		| hierarchies[0]/root/testPlan    | Plan1        |
		| hierarchies[0]/root/name        | RootSuite    |
		| hierarchies[0]/skipFolderPrefix | src/Features |

@tc:1410
Scenario: A levels hierarchy is configured
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                             | answer      |
		| Do you want to configure a hierarchy now                                             | yes         |
		| Which hierarchy type you would like to use?                                          | 'levels'    |
		| Please specify a name for the hierarchy                                              | MyHierarchy |
		| Please specify the name of the Test Plan where the hierarchy should be created       | Plan1       |
		| Please specify the path of the root Test Suite where the hierarchy should be created | RootSuite   |
	Then the configuration file should be updated with at least
		| setting                            | value       |
		| hierarchies[0]/name                | MyHierarchy |
		| hierarchies[0]/type                | levels      |
		| hierarchies[0]/root/testPlan       | Plan1       |
		| hierarchies[0]/root/name           | RootSuite   |
		| hierarchies[0]/levels[0]/condition | [any]       |

@tc:1411
Scenario: A tag hierarchy is configured
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                             | answer      |
		| Do you want to configure a hierarchy now                                             | yes         |
		| Which hierarchy type you would like to use?                                          | 'tag'       |
		| Please specify a name for the hierarchy                                              | MyHierarchy |
		| Please specify the name of the Test Plan where the hierarchy should be created       | Plan1       |
		| Please specify the path of the root Test Suite where the hierarchy should be created | RootSuite   |
		| Please specify the tag prefix to use for the hierarchy                               | suite       |
	Then the configuration file should be updated with at least
		| setting                      | value       |
		| hierarchies[0]/name          | MyHierarchy |
		| hierarchies[0]/type          | tag         |
		| hierarchies[0]/root/testPlan | Plan1       |
		| hierarchies[0]/root/name     | RootSuite   |
		| hierarchies[0]/tagPrefix     | suite       |

@tc:1412
Scenario: A single hierarchy is configured
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                       | answer      |
		| Do you want to configure a hierarchy now                                       | yes         |
		| Which hierarchy type you would like to use?                                    | 'single'    |
		| Please specify a name for the hierarchy                                        | MyHierarchy |
		| Please specify the name of the Test Plan where the hierarchy should be created | Plan1       |
		| Please specify the path of the Test Suite where the Test Cases should be added | MySuite     |
	Then the configuration file should be updated with at least
		| setting                      | value       |
		| hierarchies[0]/name          | MyHierarchy |
		| hierarchies[0]/type          | single      |
		| hierarchies[0]/root/testPlan | Plan1       |
		| hierarchies[0]/node/name     | MySuite     |

@tc:1413
Scenario: A custom hierarchy is configured
	When SpecSync upgrade is executed with answers successfully:
		| question                                                                             | answer      |
		| Do you want to configure a hierarchy now                                             | yes         |
		| Which hierarchy type you would like to use?                                          | 'custom'    |
		| Please specify a name for the hierarchy                                              | MyHierarchy |
		| Please specify the name of the Test Plan where the hierarchy should be created       | Plan1       |
		| Please specify the path of the root Test Suite where the hierarchy should be created | RootSuite   |
	Then the configuration file should be updated with at least
		| setting                           | value       |
		| hierarchies[0]/name               | MyHierarchy |
		| hierarchies[0]/type               | custom      |
		| hierarchies[0]/root/testPlan      | Plan1       |
		| hierarchies[0]/root/name          | RootSuite   |
		| hierarchies[0]/nodes[0]/name      | [any]       |
		| hierarchies[0]/nodes[0]/condition | [any]       |

@tc:1414
Scenario: No hierarchy is configured
	When SpecSync upgrade is executed with answers successfully:
		| question                                    | answer           |
		| Do you want to configure a hierarchy now    | yes              |
		| Which hierarchy type you would like to use? | I cannot decide. |
	Then the configuration file should be updated with
		| setting              | value                          |
		| configurationKey     | [any]                          |
		| compatibilityVersion | [config-compatibility-version] |
		| $schema              | [config-schema-url]            |
