﻿@hierarchies @adoSpecific @bypass-integration
Feature: Custom Test Case hierarchy in Azure DevOps

Rule: A Test Suite hierarchy can be created based on explicitly listed Test Suites with optional conditions (priority list)

@tc:1415
Scenario: A custom hierarchy is initialized with conditions
	Given there is an Azure DevOps project
	And there are scenarios in the local repository as
		| scenario | tags              |
		| S1       | @mytag            |
		| S2       | @mytag, @othertag |
		| S3       | @othertag         |
		| S4       | @tag1, @tag2      |
		| S5       |                   |
	And the synchronizer is configured as
		| setting                           | value                    |
		| hierarchies[0]/type               | custom                   |
		| hierarchies[0]/nodes[0]/name      | Custom Suite 1           |
		| hierarchies[0]/nodes[0]/condition | @mytag                   |
		| hierarchies[0]/nodes[1]/name      | Custom Suite 2           |
		| hierarchies[0]/nodes[1]/condition | @othertag                |
		| hierarchies[0]/nodes[2]/name      | Custom Suite 3           |
		| hierarchies[0]/nodes[2]/condition | @tag1 and @tag2          |
		| hierarchies[0]/nodes[3]/path      | Custom Suite 3/All other |
	When the local repository is synchronized with push
	Then the Test Suites should be created and contain the Test Cases as
		| test suite path          | test cases                 | description                                        |
		| Custom Suite 1           | Scenario: S1, Scenario: S2 | contains all with @mytag                           |
		| Custom Suite 2           | Scenario: S3               | doesn't contain S2, because matched to first suite |
		| Custom Suite 3           | Scenario: S4               | uses complex condition                             |
		| Custom Suite 3/All other | Scenario: S5               | last one without condition matches all remaining   |

Rule: The additional Test Suites are automatically ignored if only one Test Suite is specified

@tc:1416
Scenario: Custom hierarchy is used to save all Test Cases to a single Test Suite
	In case there is only one Test Suite is configured, the additional nodes are automatically ignored (like 'ignoreAdditionalNodes': false)
	Given there is an Azure DevOps project with an empty Test Suite 'Other Suite'
	And there are feature files with scenarios in the local repository as
		| feature file     | scenarios |
		| Feature1.feature | S1, S2    |
	And the synchronizer is configured as
		| setting                      | value          |
		| hierarchies[0]/type          | custom         |
		| hierarchies[0]/nodes[0]/name | Custom Suite 1 |
	When SpecSync push is executed
	Then the command should finish with success
	And the Test Suites should be created and contain the Test Cases as
		| test suite path | test cases                 |
		| Custom Suite 1  | Scenario: S1, Scenario: S2 |
